module CrmUc
  module Userable
    extend ActiveSupport::Concern

    included do
      include CrmUc::UserCenterable

      # 如果要判断user是不是独立版用户,请用User#ikcrm_user?方法
      enum user_type: {
        normal_user: 0,
        dingtalk_user: 1,
        kingdee_user: 2,
        wechat_user: 3,
        wx_user: 4
      }
      enum status: [:hide, :visible]

      if AppSettings.uc_enabled?
        enum permission: [:deny, :edit, :show]
      end

      default_scope { usable }

      scope :stateless, -> { unscope(where: [:status, :locked_at, :usable]) }
      scope :visible, -> { stateless.where(status: statuses[:visible]) }
      scope :usable, -> {
        if AppSettings.uc_enabled?
          where(usable: true)
        else
          where(status: statuses[:visible], locked_at: nil)
        end
      }

    end

    if AppSettings.wxwork?
      def phone
        self.attributes["phone"].gsub(/^lxjxc/, "") if self.attributes["phone"]
      end

      def email
        self.attributes["email"].gsub(/^lxjxc/, "") if self.attributes["email"]
      end
    end

    def parent
      return if superior_id.blank?

      self.class.stateless.find_by(id: superior_id)
    end

    def locked_at
      if AppSettings.uc_enabled?
        usable? ? nil : Time.now
      else
        self.attributes['locked_at']
      end
    end

    def ikcrm_user?
      AppSettings.ikcrm?
    end

    def wx_userid
      get_wx_userid_key = "#{uid}_to_wx_userid"
      $redis.set(get_wx_userid_key, UserCenter::Services::Wx::TransferIdServices.get_wx_userid_by_uid(organization.oid, uid), ex: 7.days) if $redis.get(get_wx_userid_key).blank?

      $redis.get(get_wx_userid_key)
    end

    def userid
      if AppSettings.wxwork?
        wx_userid
      elsif AppSettings.dingtalk?
        # TODO
        ding_user.userid
      end
    end

    def to_s
      name || email
    end

  end
end
